%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%                                %%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%      DECLIPPING MAIN FILE      %%%%%%%%%%%%%%%%%%%%%%%     
%%%%%%%%%%%%%%%%%%%%                                %%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% using toolbox LTFAT

close all;
%clear variables;

%% input file settings   
% load audio-file
fprintf('Loading audio\n')
[data, fs] = audioread('Sounds/acoustic_guitar.wav');

% normalization
maxAbsVal = max(abs(data));
if maxAbsVal < 0.95
    fprintf('Normalizing audio...')
    data = data/maxAbsVal;
    fprintf('(The original maximum was %f)\n', maxAbsVal)
end

% signal length
param.Ls = length(data);


%% settings
fprintf('Setting up the frame parameters\n')
param.theta = 0.0311687617717628;    % set (symetric) clipping threshold 

% To compare with Srdjans code, for the acoustic guitar signal set
% param.theta to [0.0311687617717628, 0.147244340814544, 0.278153830901345]
% which corresponds to input SNR 1 dB, 5 dB and 10 dB (as in Srdjans code)

% window parameters
param.w = 1024;       % window length
param.a = param.w/4;  % window shift
param.wtype = 'hamming'; % options available on: http://ltfat.github.io/doc/sigproc/firwin.html

% DFT parameters
param.F = frame('dft');
param.F.redundancy = 2;  %non-native, our own additional parameter
param.F.frana = @(insig)dft([insig; zeros(length(insig)*(param.F.redundancy-1),1)]);
param.F.frsyn = @(insig)postpad(idft(insig),length(insig)/param.F.redundancy);

% paramsolver parameters
paramsolver.s = 1;   % increment of k
paramsolver.r = 1;   % every r-th iteration increment k by s   
paramsolver.epsilon = 0.1;  % stopping criterion of termination function
paramsolver.maxit = ceil(floor(param.w*param.F.redundancy/2+1)*paramsolver.r/paramsolver.s); % maximum number of iterations

paramsolver.store_snr = 1; 
paramsolver.store_obj = 1;

param.algorithm = 'aspade'; % algorithm to compute declipping, options: 'aspade' or 'sspade'
fprintf('Setting up algorithm: %s\n', param.algorithm)


%% clipping 
fprintf('Generating clipped signal\n')
[data_clipped, param.masks] = hard_clip(data, -param.theta, param.theta); % clipping the original signal


%% Main algorithm
fprintf('Starting the %s algorithm\n', param.algorithm)

tic; 

[data_rec, snr_iter, obj_iter] = spade_segmentation(data_clipped, param, paramsolver, data);

time = toc;


%% Time & SNR evaluation

% time
fprintf('Result obtained in %4.3f seconds.\n', time);
% SNR
snr_clip = snr_n(data, data_clipped);
snr_rec = snr_n(data, data_rec);
fprintf('SDR of the clipped signal is %4.3f dB.\n', snr_clip);
fprintf('SDR of the reconstructed signal is %4.3f dB.\n', snr_rec);
fprintf('SDR improvement is %4.3f dB.\n', snr_rec-snr_clip);

